@@ -4,7 +4,7 @@ use strict;
 
 package Devel::Chitin;
 
-our $VERSION = '0.16'; # TRIAL
+our $VERSION = '0.19'; # TRIAL
 
 use Scalar::Util;
 use IO::File;
@@ -15,6 +15,7 @@ use Devel::Chitin::Eval;
 use Devel::Chitin::Stack;
 use Devel::Chitin::Location;
 use Devel::Chitin::SubroutineLocation;
+use Devel::Chitin::SubroutineReturn;
 use Devel::Chitin::Exception;
 use Devel::Chitin::OpTree;
 
@@ -87,17 +88,65 @@ sub stepover {
 }
 
 sub stepout {
+    my $self = shift;
+    if (@_) {
+        my %args = @_;
+        DB::_queue_on_sub_return($args{'cb'}) if $args{'cb'};
+    }
+
     $DB::single=0;
     $DB::step_over_depth = $DB::stack_depth - 1;
     return 1;
 }
 
-# Should support running to a subname, or file+line
 sub continue {
     $DB::single=0;
     return 1;
 }
 
+sub continue_to {
+    my $self = shift;
+    my($file, $line);
+    if (@_ == 1) {
+        # passed in a subname
+        ($file, $line) = $self->_determine_first_breakable_line_of_sub($_[0]);
+    } elsif (@_ == 2) {
+        ($file, $line) = @_;
+    }
+
+    return unless $file;  # bad args
+
+    my $rv = Devel::Chitin::Breakpoint->new(file => $file, line => $line, once => 1, code => 1);
+    $DB::single=0 if $rv;
+    return $rv;
+}
+
+sub _determine_first_breakable_line_of_sub {
+    my($self, $sub) = @_;
+
+    my $subref;
+    if (! ref($sub)) {
+        $subref = do {
+            no strict 'refs';
+            \&$sub;
+        };
+
+    } elsif (Scalar::Util::reftype($sub) eq 'CODE') {
+        $subref = $sub;
+    } else {
+        return;
+    }
+    my $cv = B::svref_2object($subref);
+    my $op = $cv->START;
+    while($op && !$op->isa('B::NULL')) {
+        if ($op->isa('B::COP')) {
+            return ($op->file, $op->line);
+        }
+        $op = $op->next;
+    }
+    return;
+}
+
 sub trace {
     local $DB::in_debugger = 1;
     my $class = shift;
@@ -196,10 +245,7 @@ sub remove_watchexpr {
 
 sub is_breakable {
     my($class, $filename, $line) = @_;
-
-    use vars qw(@dbline);
-    local(*dbline) = $main::{'_<' . $filename};
-    return $dbline[$line] + 0;   # FIXME change to == 0
+    Devel::Chitin::Actionable->is_breakable($filename, $line);
 }
 
 sub add_break {
@@ -298,6 +344,7 @@ sub file_source {
 
 my %optrees;
 our $current_sub;
+sub current_sub { $current_sub }
 sub _get_optree_for_current_sub {
     my $loc = current_location;
 
@@ -473,7 +520,12 @@ sub _do_each_client {
 
 package DB;
 
-use vars qw( %dbline @dbline );
+# If we wanted to only support 5.20 and later, these could go away and be
+# replaced by a lexical glob $dbline in subs where it's needed.
+# see details in
+# https://metacpan.org/pod/release/RJBS/perl-5.20.0/pod/perldelta.pod
+# https://rt.perl.org/Public/Bug/Display.html?id=119799
+our(%dbline, @dbline);
 
 our($stack_depth,
     $single,
@@ -579,8 +631,13 @@ sub is_breakpoint {
         foreach my $condition ( @{ $dbline{$line}->{$breakpoint_key} }) {
             next if $condition->inactive;
             my $code = $condition->code;
+
+            no warnings 'uninitialized';
             if ($code eq '1') {
                 $should_break = 1;
+            } elsif (Scalar::Util::reftype($code) eq 'CODE') {
+                local $@;
+                $should_break = eval { $code->() };
             } else {
                 ($should_break) = _eval_in_program_context($condition->code, 0);
             }
@@ -692,6 +749,15 @@ sub DB {
     }
     $subroutine ||= 'MAIN';
 
+    $current_location = Devel::Chitin::Location->new(
+        'package'   => $package,
+        filename    => $filename,
+        line        => $line,
+        subroutine  => $subroutine,
+        callsite    => scalar Devel::Chitin::Location::get_callsite(),
+        ( ref($Devel::Chitin::current_sub) ? ( subref => $Devel::Chitin::current_sub ) : () )
+    );
+
     unless ($is_initialized) {
         $is_initialized = 1;
         Devel::Chitin::_do_each_client('init');
@@ -704,14 +770,6 @@ sub DB {
     local $usercontext =
         'no strict; no warnings; ($@, $!, $^E, $,, $/, $\, $^W) = @DB::saved;' . "package $package;";
 
-    $current_location = Devel::Chitin::Location->new(
-        'package'   => $package,
-        filename    => $filename,
-        line        => $line,
-        subroutine  => $subroutine,
-        callsite    => scalar Devel::Chitin::Location::get_callsite(),
-    );
-
     $_->notify_trace($current_location) foreach values(%trace_clients);
 
     _execute_actions($filename, $line);
@@ -786,6 +844,7 @@ sub sub {
     #goto &$sub if (! $ready or $in_debugger or index($sub, 'Devel::Chitin::StackTracker') == 0 or $debugger_disabled);
 
     local $Devel::Chitin::current_sub = $sub unless $in_debugger;
+    local @Devel::Chitin::on_sub_return_queue;
 
     local @AUTOLOAD_names = @AUTOLOAD_names;
     if (index($sub, '::AUTOLOAD', -10) >= 0) {
@@ -815,10 +874,23 @@ sub sub {
     my @rv;
     if (wantarray) {
         @rv = &$sub;
+        if (@Devel::Chitin::on_sub_return_queue) {
+            my $subreturn = _trigger_on_sub_return_queue(wantarray, \@rv);
+            if (Scalar::Util::reftype($subreturn->rv) eq 'ARRAY') {
+                @rv = @{ $subreturn->rv };
+            } else {
+                @rv = ( $subreturn->rv );
+            }
+        }
     } elsif (defined wantarray) {
         $rv[0] = &$sub;
+        if (@Devel::Chitin::on_sub_return_queue) {
+            my $subreturn = _trigger_on_sub_return_queue(0, $rv[0]);
+            $rv[0] = $subreturn->rv;
+        }
     } else {
         &$sub;
+        _trigger_on_sub_return_queue(undef, undef) if @Devel::Chitin::on_sub_return_queue;
     }
 
     delete $Devel::Chitin::eval_serial{$$stack_tracker} if $stack_tracker;
@@ -826,6 +898,56 @@ sub sub {
     return wantarray ? @rv : $rv[0];
 }
 
+sub _queue_on_sub_return {
+    my $cb = shift;
+    push @Devel::Chitin::on_sub_return_queue, $cb;
+}
+
+sub _trigger_on_sub_return_queue {
+    my($wantarray, @rv) = @_;
+    my $rv = $wantarray ? \@rv : $rv[0];
+
+    my $subreturn = Devel::Chitin::SubroutineReturn->new(wantarray => $wantarray,
+                                                         rv => $rv,
+                                                         map { $_ => $previous_location->$_ } qw(package subroutine filename line));
+    $_->($subreturn) foreach @Devel::Chitin::on_sub_return_queue;
+    return $subreturn;
+}
+
+sub lsub : lvalue {
+    no strict 'refs';
+    goto &$sub if (! $ready or index($sub, 'Devel::Chitin::StackTracker') == 0 or $debugger_disabled);
+
+    local $Devel::Chitin::current_sub = $sub unless $in_debugger;
+
+    local @AUTOLOAD_names = @AUTOLOAD_names;
+    if (index($sub, '::AUTOLOAD', -10) >= 0) {
+        my $caller_pkg = substr($sub, 0, length($sub)-8);
+        my $caller_AUTOLOAD = ${ $caller_pkg . 'AUTOLOAD'};
+        unshift @AUTOLOAD_names, $caller_AUTOLOAD;
+    }
+    my $stack_tracker;
+    local @Devel::Chitin::stack_serial = @Devel::Chitin::stack_serial;
+    unless ($in_debugger) {
+        $stack_depth++;
+        $stack_tracker = _new_stack_tracker(_allocate_sub_serial());
+
+        my $subname = $sub;
+        if (ref $sub) {
+            my $cv = B::svref_2object($sub);
+            my $gv = $cv->GV;
+            if (my $name = $gv->NAME) {
+                my $package = $gv->STASH->NAME;
+                $subname = join('::', $package, $name);
+            }
+        }
+
+        push(@Devel::Chitin::stack_serial, [ $subname, $$stack_tracker]);
+    }
+
+    &$sub;
+}
+
 sub _new_stack_tracker {
     my $token = shift;
     my $self = bless \$token, 'Devel::Chitin::StackTracker';
@@ -1013,6 +1135,19 @@ after that subroutine call returns.
 Continue running the debugged program until the current subroutine returns
 or until the next breakpoint, whichever comes first.
 
+=item CLIENT->stepout(cb => $subref);
+
+This form of stepout() allows registering a callback to be invoked when the
+current subroutine returns.  The callback's first argument is an instance of
+L<Devel::Chitin::SubroutineReturn>, which provides access to the location
+the function is returning from, its wantarray status, and the function's
+return value.  The C<rv> property of the SubroutineReturn object is mutable,
+and actually changes the value being returned from the function.
+
+Callbacks are invoked in the order they are queued, and a return value changed
+in this way is presented as the return value to the next callback.  The final
+callback gets the last say about the ultimate return value from the function.
+
 =item CLIENT->continue()
 
 Continue running the debugged program until the next breakpoint.
